package com.ruoyi.web.client;

import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.function.BiConsumer;

/**
 * EMQX 物联网MQTT客户端工具类
 * 功能：连接管理、自动重连、消息发布/订阅、QoS支持、遗嘱消息、SSL支持等
 */
@Slf4j
//@Component
public class EmqxIotClient implements MqttCallbackExtended {

    // ========== 配置参数 ==========
    @Value("${emqx.broker-url:tcp://127.0.0.1:1883}")
    private String brokerUrl;

    @Value("${emqx.username:}")
    private String username;

    @Value("${emqx.password:}")
    private String password;

    @Value("${emqx.client-id-prefix:iot-client}")
    private String clientIdPrefix;

    @Value("${emqx.auto-reconnect:true}")
    private boolean autoReconnect;

    @Value("${emqx.connection-timeout:10}")
    private int connectionTimeout;

    @Value("${emqx.keep-alive-interval:60}")
    private int keepAliveInterval;

    @Value("${emqx.clean-session:false}")
    private boolean cleanSession;

    @Value("${emqx.max-inflight:1000}")
    private int maxInflight;

    @Value("${emqx.will.topic:}")
    private String willTopic;

    @Value("${emqx.will.payload:}")
    private String willPayload;

    @Value("${emqx.will.qos:1}")
    private int willQos;

    @Value("${emqx.will.retained:false}")
    private boolean willRetained;

    // ========== 运行时组件 ==========
    private IMqttAsyncClient mqttClient;
    private final ConcurrentHashMap<String, BiConsumer<String, String>> topicCallbacks = new ConcurrentHashMap<>();
    private ScheduledExecutorService reconnectExecutor;
    private final MemoryPersistence persistence = new MemoryPersistence();
    private volatile boolean isConnected = false;

    // ========== 初始化方法 ==========
    @PostConstruct
    public void init() {
        connect();
        this.reconnectExecutor = Executors.newSingleThreadScheduledExecutor();
    }

    @PreDestroy
    public void destroy() {
        disconnect();
        if (reconnectExecutor != null) {
            reconnectExecutor.shutdownNow();
        }
    }

    // ========== 连接管理 ==========
    public synchronized void connect() {
        if (isConnected) {
            return;
        }

        try {
            String clientId = clientIdPrefix + "-" + UUID.randomUUID().toString().substring(0, 8);
            mqttClient = new MqttAsyncClient(brokerUrl, clientId, persistence);

            MqttConnectOptions options = new MqttConnectOptions();
            options.setUserName(username);
            options.setPassword(password.toCharArray());
            options.setAutomaticReconnect(autoReconnect);
            options.setConnectionTimeout(connectionTimeout);
            options.setKeepAliveInterval(keepAliveInterval);
            options.setCleanSession(cleanSession);
            options.setMaxInflight(maxInflight);

            // 配置遗嘱消息
            if (willTopic != null && !willTopic.isEmpty()) {
                options.setWill(willTopic, willPayload.getBytes(), willQos, willRetained);
            }

            // SSL配置（示例）
            /*
            if (useSSL) {
                options.setSocketFactory(sslContext.getSocketFactory());
            }
            */

            mqttClient.setCallback(this);
            mqttClient.connect(options, null, new IMqttActionListener() {
                @Override
                public void onSuccess(IMqttToken asyncActionToken) {
                    isConnected = true;
                    log.info("EMQX连接成功, clientId: {}", clientId);
                    // 重新订阅之前注册的主题
                    resubscribeTopics();
                }

                @Override
                public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
                    log.error("EMQX连接失败", exception);
                    scheduleReconnect();
                }
            });

        } catch (MqttException e) {
            log.error("EMQX客户端初始化异常", e);
            scheduleReconnect();
        }
    }

    public synchronized void disconnect() {
        if (!isConnected || mqttClient == null) {
            return;
        }

        try {
            mqttClient.disconnect().waitForCompletion();
            isConnected = false;
            log.info("EMQX连接已断开");
        } catch (MqttException e) {
            log.error("EMQX断开连接异常", e);
        }
    }

    private void scheduleReconnect() {
        if (!autoReconnect || reconnectExecutor == null || reconnectExecutor.isShutdown()) {
            return;
        }

        reconnectExecutor.schedule(() -> {
            log.info("尝试重新连接EMQX...");
            connect();
        }, 5, TimeUnit.SECONDS);
    }

    // ========== 消息发布 ==========
    public void publish(String topic, String payload) throws MqttException {
        publish(topic, payload, 1, false);
    }

    public void publish(String topic, String payload, int qos, boolean retained) throws MqttException {
        if (!isConnected) {
            throw new MqttException(MqttException.REASON_CODE_CLIENT_NOT_CONNECTED);
        }

        MqttMessage message = new MqttMessage(payload.getBytes());
        message.setQos(qos);
        message.setRetained(retained);

        mqttClient.publish(topic, message, null, new IMqttActionListener() {
            @Override
            public void onSuccess(IMqttToken asyncActionToken) {
                log.debug("消息发布成功, topic: {}, qos: {}", topic, qos);
            }

            @Override
            public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
                log.error("消息发布失败, topic: {}", topic, exception);
            }
        });
    }

    // ========== 消息订阅 ==========
    public void subscribe(String topic, BiConsumer<String, String> messageHandler) throws MqttException {
        subscribe(topic, 1, messageHandler);
    }

    public void subscribe(String topic, int qos, BiConsumer<String, String> messageHandler) throws MqttException {
        if (!isConnected) {
            throw new MqttException(MqttException.REASON_CODE_CLIENT_NOT_CONNECTED);
        }

        topicCallbacks.put(topic, messageHandler);
        mqttClient.subscribe(topic, qos, null, new IMqttActionListener() {
            @Override
            public void onSuccess(IMqttToken asyncActionToken) {
                log.info("订阅成功, topic: {}, qos: {}", topic, qos);
            }

            @Override
            public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
                log.error("订阅失败, topic: {}", topic, exception);
                topicCallbacks.remove(topic);
            }
        });
    }

    public void unsubscribe(String topic) throws MqttException {
        if (!isConnected) {
            throw new MqttException(MqttException.REASON_CODE_CLIENT_NOT_CONNECTED);
        }

        mqttClient.unsubscribe(topic);
        topicCallbacks.remove(topic);
        log.info("取消订阅, topic: {}", topic);
    }

    private void resubscribeTopics() {
        if (topicCallbacks.isEmpty()) {
            return;
        }

        topicCallbacks.forEach((topic, handler) -> {
            try {
                mqttClient.subscribe(topic, 1);
            } catch (MqttException e) {
                log.error("重新订阅失败, topic: {}", topic, e);
            }
        });
    }

    // ========== MQTT回调接口 ==========
    @Override
    public void connectComplete(boolean reconnect, String serverURI) {
        isConnected = true;
        log.info("MQTT连接{}成功: {}", reconnect ? "重连" : "建立", serverURI);
    }

    @Override
    public void connectionLost(Throwable cause) {
        isConnected = false;
        log.warn("MQTT连接丢失", cause);
        if (autoReconnect) {
            scheduleReconnect();
        }
    }

    @Override
    public void messageArrived(String topic, MqttMessage message) throws Exception {
        String payload = new String(message.getPayload());
        log.debug("收到消息, topic: {}, qos: {}, payload: {}", topic, message.getQos(), payload);

        BiConsumer<String, String> handler = topicCallbacks.get(topic);
        if (handler != null) {
            try {
                handler.accept(topic, payload);
            } catch (Exception e) {
                log.error("消息处理异常, topic: {}", topic, e);
            }
        } else {
            log.warn("未找到对应的消息处理器, topic: {}", topic);
        }
    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken token) {
        // 消息发布完成回调
    }

    // ========== 状态检查 ==========
    public boolean isConnected() {
        return isConnected && mqttClient != null && mqttClient.isConnected();
    }

    public String getClientId() {
        return mqttClient != null ? mqttClient.getClientId() : null;
    }
}