package com.ruoyi.web.service.impl;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.web.domain.DeviceInfo;
import com.ruoyi.web.service.IDeviceInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.web.mapper.DeviceModelMapper;
import com.ruoyi.web.domain.DeviceModel;
import com.ruoyi.web.service.IDeviceModelService;
import lombok.RequiredArgsConstructor;

/**
 * 设备型号Service业务层处理
 *
 * @author cx
 * @date 2025-04-05
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class DeviceModelServiceImpl extends ServiceImpl<DeviceModelMapper, DeviceModel> implements IDeviceModelService
{
    private final DeviceModelMapper deviceModelMapper;
    private final IDeviceInfoService deviceInfoService;

    /**
     * 查询设备型号
     *
     * @param id 设备型号主键
     * @return 设备型号
     */
    @Override
    public DeviceModel selectDeviceModelById(String id)
    {
        return getById(id);
    }

    /**
     * 查询设备型号列表
     *
     * @param deviceModel 设备型号
     * @return 设备型号
     */
    @Override
    public List<DeviceModel> selectDeviceModelList(DeviceModel deviceModel)
    {
        LambdaQueryWrapper<DeviceModel> wrapper = new LambdaQueryWrapper<>();
        wrapper.like(deviceModel.getDeviceModel() != null, DeviceModel::getDeviceModel, deviceModel.getDeviceModel());
        wrapper.eq(deviceModel.getDevicePic() != null, DeviceModel::getDevicePic, deviceModel.getDevicePic());
        return list(wrapper);
    }

    /**
     * 新增设备型号
     *
     * @param deviceModel 设备型号
     * @return 结果
     */
    @Override
    public int insertDeviceModel(DeviceModel deviceModel)
    {
        validModelCode(deviceModel);
        LoginUser user = SecurityUtils.getLoginUser();
        deviceModel.setCreateBy(user.getUsername());
        deviceModel.setUpdateBy(user.getUsername());
        return save(deviceModel) ? 1 : 0;
    }

    private void validModelCode(DeviceModel deviceModel) {
        LambdaQueryWrapper<DeviceModel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(DeviceModel::getDeviceModel, deviceModel.getDeviceModel());
        List<DeviceModel> list = this.list(queryWrapper);
        if (!list.isEmpty()) {
            throw new ServiceException("该设备型号已存在,不能重复录入相同型号");
        }
    }

    /**
     * 修改设备型号
     *
     * @param deviceModel 设备型号
     * @return 结果
     */
    @Override
    public int updateDeviceModel(DeviceModel deviceModel)
    {
        String username = SecurityUtils.getUsername();
        deviceModel.setUpdateBy(username);
        return updateById(deviceModel) ? 1 : 0;
    }

    /**
     * 批量删除设备型号
     *
     * @param ids 需要删除的设备型号主键
     * @return 结果
     */
    @Override
    public int deleteDeviceModelByIds(String[] ids)
    {
        List<DeviceModel> deviceModels = this.listByIds(Arrays.asList(ids));
        List<String> deviceCodeList = deviceModels.stream().map(DeviceModel::getDeviceModel).collect(Collectors.toList());
        LambdaQueryWrapper<DeviceInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(DeviceInfo::getDeviceCode, deviceCodeList.toArray());
        List<DeviceInfo> infoList = deviceInfoService.list(queryWrapper);
        if (!infoList.isEmpty()) {
            log.warn("设备型号 {} 已绑定设备，无法删除", JSON.toJSONString(deviceCodeList));
            return 0;
        }
        return removeByIds(Arrays.asList(ids)) ? 1 : 0;
    }

    /**
     * 删除设备型号信息
     *
     * @param id 设备型号主键
     * @return 结果
     */
    @Override
    public int deleteDeviceModelById(String id)
    {
        return removeById(id) ? 1 : 0;
    }
}